
import java.net.*;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.*;
import com.bullhorn.apiservice.*;
import com.bullhorn.apiservice.query.*;
import com.bullhorn.apiservice.result.*;
import com.bullhorn.apiservice.session.*;
import com.bullhorn.entity.candidate.CandidateDto;
import com.bullhorn.entity.job.JobOrderDto;
import com.bullhorn.entity.note.NoteDto;
import com.bullhorn.entity.client.*;

public class Notes {

    private static final QName SERVICE_NAME = new QName("http://apiservice.bullhorn.com/", "ApiService");

    public static void main(String[] args) throws Exception {

        String username = "yourusername";
        String password = "yourpassword";
        String apiKey = "yourapikey";
        String wsdlUrl = "https://api.bullhornstaffing.com/webservices-1.1/?wsdl"; 
        
       
        // obtain the API web service
        URL serviceUrl = new URL(ApiService_Service.class.getResource("."), wsdlUrl);
        ApiService service = new ApiService_Service(serviceUrl, SERVICE_NAME).getApiServicePort();

        // Login to obtain an API session
        ApiSession currentSession = service.startSession(username, password, apiKey);

        XMLGregorianCalendar gregDate;
        gregDate = DatatypeFactory.newInstance().newXMLGregorianCalendar();
        gregDate.setDay(12);
        gregDate.setMonth(12);
        gregDate.setYear(2010);
        gregDate.setHour(8);
        gregDate.setMinute(30);
        gregDate.setSecond(0);
        
        // Create a new note
        NoteDto noteDto = new NoteDto();
        noteDto.setAction("sample");
        noteDto.setCommentingPersonID(currentSession.getUserId());
        noteDto.setComments("This is a new note.");
        // Change this based on your account
        noteDto.setPersonReferenceID(6521);
        
        ApiSaveResult results = service.save(currentSession, noteDto);
        currentSession = results.getSession();
        
        NoteDto SavedNoteDto = (NoteDto)results.getDto(); 
        
        // Associate the note with a candidate
        service.addNoteReference(currentSession, SavedNoteDto.getNoteID(), "Candidate", 6521);
        System.out.println("Done");
        
        DtoQuery candidateQuery = new DtoQuery();
        candidateQuery.setEntityName("Candidate");
        candidateQuery.setMaxResults(10);
        candidateQuery.setWhere("FirstName = 'Delvan'");
        ApiQueryResult queryCandidateResult = service.query(currentSession, candidateQuery);
        currentSession = queryCandidateResult.getSession();

        // Add notes
        if (!queryCandidateResult.getIds().isEmpty())
        {
          for (int j =0; j< queryCandidateResult.getIds().size() ; j++)
          {
        	  ApiFindResult candidate = service.find(currentSession, "Candidate", queryCandidateResult.getIds().get(j));
        	  currentSession = candidate.getSession();
              CandidateDto thisCandidate = (CandidateDto)candidate.getDto();

		        // Create a new note
		        NoteDto newnoteDto = new NoteDto();
		        newnoteDto.setCommentingPersonID(currentSession.getUserId());
		        newnoteDto.setComments("This is the best note ever.");
		        newnoteDto.setDateAdded(gregDate);
		        newnoteDto.setIsDeleted(false);
		        newnoteDto.setPersonReferenceID(thisCandidate.getUserID());
		        
		        ApiSaveResult results1 = service.save(currentSession, newnoteDto);
		        currentSession = results1.getSession();
		        
		        NoteDto SavedNoteDto1 = (NoteDto)results.getDto(); 
		       
		        // Associate the note with a candidate
		        service.addNoteReference(currentSession, SavedNoteDto1.getNoteID(), "Candidate", thisCandidate.getUserID());
		        
		        

		        // Query a job
		        DtoQuery jobQuery = new DtoQuery();
		        jobQuery.setEntityName("JobOrder");
		        jobQuery.setMaxResults(20);
		        jobQuery.setWhere("isOpen = 1");
		        ApiQueryResult queryResult = service.query(currentSession, jobQuery);
		        currentSession = queryResult.getSession();

		        if (!queryResult.getIds().isEmpty())
		        {
		          for (int i =0; i< queryResult.getIds().size() ; i++)
		          {
		        	  ApiFindResult jobOrder = service.find(currentSession, "JobOrder", queryResult.getIds().get(i));
		        	  currentSession = jobOrder.getSession();
		        	  JobOrderDto thisJobOrder = (JobOrderDto)jobOrder.getDto();

		        	  // Associate the note with a job order
		        	  service.addNoteReference(currentSession, SavedNoteDto.getNoteID(), "JobOrder", thisJobOrder.getJobOrderID());
		          }
		        }
		        
		        // Retrieve the client contact id before running this. It is hard coded here, change this based on your account
		        // Associate the note with a client contact
		        service.addNoteReference(currentSession, SavedNoteDto.getNoteID(), "ClientContact", 6600);
		        
          }
        } 
        
        
        // Remove note reference for a candidate
        if (!queryCandidateResult.getIds().isEmpty())
        {
          for (int j =0; j< queryCandidateResult.getIds().size() ; j++)
          {
        	  ApiFindResult candidate = service.find(currentSession, "Candidate", queryCandidateResult.getIds().get(j));
        	  currentSession = candidate.getSession();
              CandidateDto thisCandidate = (CandidateDto)candidate.getDto();
              
              ApiGetEntityNotesResult noteResult = service.getEntityNotes(currentSession, "Candidate", thisCandidate.getUserID());
              currentSession = noteResult.getSession();
              if (!noteResult.getIds().isEmpty())
		        {
		          for (int k =0; k < noteResult.getIds().size() ; k++)
		          {
                	  
		        	  // Remove note references
		        	  service.removeNoteReference(currentSession, noteResult.getIds().get(k), "Candidate", thisCandidate.getUserID());
		          }
		        }
        
               
		        // Query a job
		        DtoQuery jobQuery = new DtoQuery();
		        jobQuery.setEntityName("JobOrder");
		        jobQuery.setMaxResults(20);
		        jobQuery.setWhere("isOpen = 1");
		        ApiQueryResult queryJobResult = service.query(currentSession, jobQuery);
		        currentSession = queryJobResult.getSession();

		        if (!queryJobResult.getIds().isEmpty())
		        {
		          for (int l =0; l< queryJobResult.getIds().size() ; l++)
		          {
		        	  ApiFindResult jobOrder = service.find(currentSession, "JobOrder", queryJobResult.getIds().get(l));
		        	  currentSession = jobOrder.getSession();
		        	  JobOrderDto thisJobOrder = (JobOrderDto)jobOrder.getDto();

		        	  ApiGetEntityNotesResult noteResult1 = service.getEntityNotes(currentSession, "JobOrder", thisJobOrder.getJobOrderID());
		              currentSession = noteResult1.getSession();
		              if (!noteResult1.getIds().isEmpty())
				        {
				          for (int k =0; k < noteResult1.getIds().size() ; k++)
				          {
		                	  // Remove note references
				        	  service.removeNoteReference(currentSession, noteResult1.getIds().get(k), "JobOrder", thisJobOrder.getJobOrderID());
				          }
				        }
		          }
		        }
		        

		        // Query a job
		        DtoQuery clientContactQuery = new DtoQuery();
		        clientContactQuery.setEntityName("ClientContact");
		        clientContactQuery.setMaxResults(20);
		        clientContactQuery.setWhere("isDeleted = 0");
		        ApiQueryResult queryClientContactResult = service.query(currentSession, clientContactQuery);
		        currentSession = queryClientContactResult.getSession();

		        if (!queryClientContactResult.getIds().isEmpty())
		        {
		          for (int l =0; l< queryClientContactResult.getIds().size() ; l++)
		          {
		        	  ApiFindResult clientContact = service.find(currentSession, "ClientContact", queryClientContactResult.getIds().get(l));
		        	  currentSession = clientContact.getSession();
		        	  ClientContactDto thisClientContact = (ClientContactDto)clientContact.getDto();

		        	  ApiGetEntityNotesResult noteResult1 = service.getEntityNotes(currentSession, "ClientContact", thisClientContact.getUserID());
		              currentSession = noteResult1.getSession();
		              if (!noteResult1.getIds().isEmpty())
				        {
				          for (int k =0; k < noteResult1.getIds().size() ; k++)
				          {
				        	  // Retrieve the client contact id before running this. 
				        	  service.removeNoteReference(currentSession, noteResult1.getIds().get(k), "ClientContact", thisClientContact.getUserID());
				          }
				        }
		        
		          }
		        }
	        }
	    } 
        
        // Candidate ID is hard coded here, change this based on your account
        ApiGetEntityNotesResult resultNotes = service.getEntityNotesWhere(currentSession, "Candidate", 6521, "note.action = 'sample'");
        // Refresh the session
        currentSession = resultNotes.getSession();
        
        for (int i = 0; i < resultNotes.getIds().size(); i++ )
        {
        	  ApiFindResult findNoteResult = service.find(currentSession, "Note", resultNotes.getIds().get(i));
        	  // Refresh the session
        	  currentSession = findNoteResult.getSession();
        	  NoteDto thisNote = (NoteDto)findNoteResult.getDto();
            // Print candidate's information to console
            System.out.println("Found note using getEntityNotesWhere() operation: " +thisNote.getComments());
        }
        
        ApiGetEntityNotesResult resultNotes1 = service.getEntityNotes(currentSession, "Candidate", 6521);
        // Refresh the session
        currentSession = resultNotes1.getSession();
        
        for (int i = 0; i < resultNotes1.getIds().size(); i++ )
        {
        	  ApiFindResult findNoteResult = service.find(currentSession, "Note", resultNotes1.getIds().get(i));
        	  // Refresh the session
        	  currentSession = findNoteResult.getSession();
        	  NoteDto thisNote = (NoteDto)findNoteResult.getDto();
            // Print candidate's information to console
            System.out.println("Found note using getEntityNotes() operation: " +thisNote.getComments());
        }
  
      }
       
    }
